'*****************************************************************************
'*  Name    : Firewing                                                       *
'*  Author  : David John Barker                                              *
'*  Notice  : Copyright (c) 2014 Mecanique                                   *
'*          : All Rights Reserved                                            *
'*  Date    : 15/03/2014                                                     *
'*  Version : 1.1                                                            *
'*  Notes   : 1.1 Corrected CCPxCON assignment error                         *
'*          : 1.0 Release version                                            *
'*          :                                                                *
'*****************************************************************************
Module Firewing

' firewing board pin aliases
' http://www.firewing.info/downloads/pdf/pic-pin-map-r2-pic18.pdf
Public Dim A0 As PORTB.0
Public Dim A1 As PORTB.3
Public Dim A2 As PORTA.0
Public Dim A3 As PORTA.1
Public Dim A4 As PORTB.2
Public Dim A5 As PORTB.1
Public Dim ARef As PORTA.3
Public Dim D0 As PORTC.7
Public Dim D1 As PORTC.6
Public Dim D2 As PORTB.7
Public Dim D3 As PORTB.6
Public Dim D4 As PORTC.0
Public Dim D5 As PORTC.1
Public Dim D6 As PORTC.2
Public Dim D7 As PORTB.5
Public Dim D8 As PORTB.4
Public Dim D9 As PORTA.2
Public Dim D10 As PORTA.5
Public Dim D11 As PORTC.5
Public Dim D12 As PORTC.4
Public Dim D13 As PORTC.3
Public Dim D14 As PORTA.4

' Adc clock period...
Public Const IsFosc2 = %000
Public Const IsFosc4 = %100
Public Const IsFosc8 = %001
Public Const IsFosc16 = %101
Public Const IsFosc32 = %010
Public Const IsFosc64 = %110
Public Const IsFoscFrc = %011   

' Adc aquisition time...
Public Const Is0Tad = %000
Public Const Is2Tad = %001
Public Const Is4Tad = %010
Public Const Is6Tad = %011
Public Const Is8Tad = %100
Public Const Is12Tad = %101
Public Const Is16Tad = %110
Public Const Is20Tad = %111 

' Adc voltage reference...
Public Const IsInternal = 0
Public Const IsExternal = 1

' Adc private aliases...
Dim _adfm As ADCON2.7      ' right or left justify (default is right)
Dim _adcs2 As ADCON2.2     ' ADC clock select bit (2)
Dim _adcs1 As ADCON2.1     ' ADC clock select bit (1)
Dim _adcs0 As ADCON2.0     ' ADC clock select bit (0)
Dim _acqt2 As ADCON2.5     ' ADC acquisition time bit (2)
Dim _acqt1 As ADCON2.4     ' ADC acquisition time bit (1)
Dim _acqt0 As ADCON2.3     ' ADC acquisition time bit (0)

' Pwm private aliases...
Dim _timerOn As T2CON.2    ' use timer 2
Dim _pwmPin_01 As PORTC.1  ' first pwm pin  (D5)
Dim _pwmPin_02 As PORTC.2  ' second pwm pin (D6)
Dim _maxDuty As Word       ' holds the calculated maximum suty cycle

'****************************************************************************
'* Name    : AdcSetConversionTime                                           *
'* Purpose : Set the conversion time                                        *
'****************************************************************************
Public Sub AdcSetConversionTime(period As Byte)
   _adcs2 = period.2
   _adcs1 = period.1
   _adcs0 = period.0
End Sub 

'****************************************************************************
'* Name    : AdcSetAquisitionTime                                           *
'* Purpose : Set ADC aquisition time                                        *
'****************************************************************************
Public Sub AdcSetAquisitionTime(time As Byte)
   _acqt2 = time.2
   _acqt1 = time.1
   _acqt0 = time.0
End Sub

'****************************************************************************
'* Name    : AdcSetReference                                                *
'* Purpose : Set ADC voltage reference                                      *
'****************************************************************************
Public Sub AdcSetReference(ref As Byte)
   If ref = IsInternal Then
      ADCON1 = 0
   Else
      ADCON1 = %00000100
   EndIf
End Sub

'****************************************************************************
'* Name    : AdcRead                                                        *
'* Purpose : Read a Adc value from pins A0...A5                             *
'****************************************************************************
Public Function AdcRead(ByRef pin As Bit) As Word
   Dim portAddress As Word
   Dim portBit As Byte
   portAddress = AddressOf(pin)
   portBit = BitOf(pin, false)

   ' if pin belongs to PORTB...
   If portAddress = AddressOf(PORTB) Then
   
      ' select pin...
      Select portBit
         Case 0 : ' A0
            TRISB.0 = 1
            ANSELB.0 = 1
            ADCON0 = $33
         Case 1 : ' A5
            TRISB.1 = 1
            ANSELB.1 = 1
            ADCON0 = $2B
         Case 2 : ' A4
            TRISB.2 = 1
            ANSELB.2 = 1
            ADCON0 = $23 
         Case 3 : ' A1
            TRISB.3 = 1
            ANSELB.3 = 1
            ADCON0 = $27 
       EndSelect
       
    ' if pin belongs to PORTA...
    ElseIf portAddress = AddressOf(PORTA) Then
    
       ' select pin...
       Select portBit
          Case 0 ' A2
             TRISA.0 = 1
             ANSELA.0 = 1
             ADCON0 = $03  
          Case 1 ' A3
             TRISA.1 = 1
             ANSELA.1 = 1
             ADCON0 = $07 
      EndSelect
   EndIf  
   
   ' wait for conversion...
   Repeat
   Until ADCON0.1 = 0
   
   ' read value...
   AdcRead.Byte1 = ADRESH
   AdcRead.Byte0 = ADRESL
End Function

'****************************************************************************
'* Name    : PwmMaxDuty                                                     *
'* Purpose : Returns the max duty for a set frequency                       *
'****************************************************************************
Public Function PwmMaxDuty() As _maxDuty
End Function  
   
'****************************************************************************
'* Name    : PwmOpen                                                        *
'* Purpose : Open a Pwm port                                                *
'****************************************************************************
Public Sub PwmOpen(ByRef pin As Bit)
   If AddressOf(pin) <> AddressOf(_pwmPin_01) Then 
      Exit
   EndIf
   If BitOf(pin, false) = 2 Then
      CCP1CON = $0C
      Output(_pwmPin_01)
      _timerOn = 1
   ElseIf BitOf(pin, false) = 1 Then
      CCP2CON = $0C
      Output(_pwmPin_02) 
      _timerOn = 1 
   EndIf      
End Sub

'****************************************************************************
'* Name    : PwmClose                                                       *
'* Purpose : Close Pwm port                                                 *
'****************************************************************************
Public Sub PwmClose(ByRef pin As Bit)
   If AddressOf(pin) <> AddressOf(_pwmPin_01) Then 
      Exit
   EndIf
   If BitOf(pin, false) = 2 Then
      CCP1CON = 0
      Input(_pwmPin_01)
   ElseIf BitOf(pin, false) = 1 Then
      CCP2CON = 0
      Input(_pwmPin_02) 
   EndIf      
End Sub
   
'****************************************************************************
'* Name    : PwmWrite                                                       *
'* Purpose : Write duty to open Pwm port                                    *
'****************************************************************************
Public Sub PwmWrite(ByRef pin As Bit, duty As Word)
   If AddressOf(pin) <> AddressOf(_pwmPin_01) Then 
      Exit
   EndIf
   If BitOf(pin, false) = 2 Then
      CCP1CON.5 = duty.1
      CCP1CON.4 = duty.0   
      CCPR1L = duty >> 2
   ElseIf BitOf(pin, false) = 1 Then
      CCP2CON.5 = duty.1
      CCP2CON.4 = duty.0   
      CCPR2L = duty >> 2
   EndIf 
End Sub

'****************************************************************************
'* Name    : PwmSetFrequency                                                *
'* Purpose : Set Pwm frequency - reurns true if frequency can be set, false *
'*         : otherwise - use PwmMaxDuty() to determine duty range for a     *
'*         : given frequency                                                *
'****************************************************************************
' set pwm frequency...   
Public Function PwmSetFrequency(frequency As LongWord) As Boolean
   Const fosc As LongWord = _clock * 1000000
   Dim prConst As Word 
   Dim pr2Value As Word 
   Dim prescale As Word 

   prConst = Fosc / frequency / 4
   prescale = 1
   
   ' loop through all the valid prescalers...
   Repeat
      pr2Value = prConst / prescale - 1        ' calculate a PR2 value    
      If pr2Value < 256 Then                   ' if it is a valid value, then...
         _maxDuty = (pr2Value + 1) * 4         ' determine maximum duty... 
         PR2 = pr2Value                        ' initialise PR2                 
         Select prescale                       ' configure T2CON prescale
            Case 1  : prescale = %00000000     ' prescale 1
            Case 4  : prescale = %00000001     ' prescale 4
            Case 16 : prescale = %00000011     ' prescale 16
         End Select
         T2CON = (T2CON And 252) Or prescale  ' load prescaler value
         result = true
         Exit
      EndIf   
      prescale = prescale * 4
   Until prescale > 16 
   result = false
End Function

' init Adc, if required...
#ifused (AdcRead) 
   ADCON2.7 = 1  ' right justify 
   AdcSetConversionTime(IsFosc64)
   AdcSetReference(IsInternal) 
   AdcSetAquisitionTime(Is4Tad)
#endused 

' init Pwm, if required...
#ifused(Open)  
   _maxDuty = 0                     
   PwmSetFrequency(3906)
#endused

End Module
